*******************************************************************************;
** SAS macro SAS_name vs. 0.3, 19 Mar 2003, (C) Jim Groeneveld, Netherlands   *;
*******************************************************************************;
**                                                                            *;
*; %MACRO   SAS_name   /* Checking validity of SAS names, like SCL's SASNAME  */
/* -------------------------------------------------------------------------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  Name    =           /* Variable, library, dataset, format name to check  R */
,  Type    =           /* Type of SAS name to check, currently supporting:  R */
                       /*   Variable, Array, DataSet, View, Catalog, Index,   */
                       /*   Entry, DsWindow, Macro, MacVar, McWindow,         */
                       /*   SCLvar, VarLabel, GenerDs, Informat, Function,    */
                       /*   CallRoutine, Format, Engine, LibName, FileName    */
,  CalledBy=SAS_name   /* Macro name of macro or program for err0r report   O */
,  Version = 0.3       /* forced version specification for version control  O */
,  ForMyUse=DO_NOT_USE /* argument for the author's use only, do not use it ! */
)/ DES     =             'SAS_name check, like SCL`s SASname'     /* STORE */  ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : SAS_NAME.SAS                                                  *;
%*                                                                            *;
%* Purpose    : Check validity of SAS names, names of variables, libraries,   *;
%*              datasets, formats and so on, similar to the SCL function      *;
%*              SASname. Validity check includes varying name lengths.        *;
%*                                                                            *;
%* Programmer : Jim Groeneveld                                                *;
%* Date       : 19 Mar 2003                                                   *;
%* Version    : 0.3                                                           *;
%*                                                                            *;
%* SAS version: 6.12 - 8.x under Windows 95 - 2K                              *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Summary                                                                    *;
%* -------                                                                    *;
%* Macro SAS_name checks for the validity of several kinds of SAS names,      *;
%* based on various characteristics (lengths, characters) and SAS version     *;
%* dependent. If a name has been verified correctly the macro returns the     *;
%* text Right, while otherwise it generally returns the text Wrong,       *;
%* only if the type does not match one of the explicit built-in types the     *;
%* result value returned may become Unmatched Type.                         *;
%*                                                                            *;
%* Examples                                                                   *;
%* --------                                                                   *;
%*                                                                            *;
/* %PUT ValidNam = %SAS_name (Name = ValidNam, Type = variable);              */
/* %PUT 1alidNam = %SAS_name (Name = 1alidNam, Type = variable);              */
/* %PUT ValidName = %SAS_name (Name = ValidName, Type = variable);            */
/* %PUT Val-Name = %SAS_name (Name = Val-Name, Type = variable);              */
/* %PUT ValidNam = %SAS_name (Name = ValidNam, Type = dataset);               */
/* %PUT Val Name = %SAS_name (Name = Val Name, Type = variable);              */
/* %PUT V2+v3=V4 = %SAS_name (Name = V2+v3=v4, Type = variable);              */
/* %PUT WrFormat = %SAS_name (Name = WrFormat, Type = format  );              */
/* %PUT OutFormat = %SAS_name (Name = OutFormat, Type = format);              */
/* %PUT InFormat = %SAS_name (Name = Informat, Type = informat);              */
/* %PUT $Format  = %SAS_name (Name = $Format , Type = format  );              */
/* %PUT $In12Fmt = %SAS_name (Name = $In12Fmt, Type = informat);              */
/* %PUT $In_Fmt  = %SAS_name (Name = $In_Fmt , Type = informat);              */
/* %PUT LastNum4 = %SAS_name (Name = LastNum4, Type = format  );              */
/* %PUT $Format4 = %SAS_name (Name = $Format4, Type = format  );              */
/* %PUT InFmt5   = %SAS_name (Name = InFmt5  , Type = informat);              */
/* %PUT $InFmt5  = %SAS_name (Name = $InFmt5 , Type = informat);              */
/* %PUT $        = %SAS_name (Name = $       , Type = informat);              */
/* %PUT F        = %SAS_name (Name = F       , Type = informat);              */
/* %PUT 6        = %SAS_name (Name = 6       , Type = informat);              */
/* %PUT 6F       = %SAS_name (Name = 6F      , Type = informat);              */
/* %PUT $F       = %SAS_name (Name = $F      , Type = informat);              */
/* %PUT $6F      = %SAS_name (Name = $6F     , Type = informat);              */
/* %PUT AllRight = %SAS_name (Name = AllRight, Type = unknown);               */
%*                                                                            *;
%* Used auxiliary macros:                                                     *;
%* ----------------------                                                     *;
%* = internal: - [none]                                                       *;
%* = attached: - [none]                                                       *;
%* = external: - [none]                                                       *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of SAS_name                                                        *;
%* ===================                                                        *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0       6 Jun 2002  initial working release                              *;
%* 0.1       8 Nov 2002  mainly cosmetic changes,                             *;
%*                       added CalledBy argument,                             *;
%*                       added version control                                *;
%* 0.2      15 Nov 2002  added SAS v9 support, incl. anticipated future ones  *;
%* 0.3      19 Mar 2003  added much more discriminative knowledge on V7,V8,V9 *;
%*                       (not yet applicable to 'AnyChars'N name constructs)  *;
%*                                                                            *;
%* Future                                                                     *;
%* ======                                                                     *;
%* - compatibility with 'AnyChars'N name constructs                           *;
%*                                                                            *;
%******************************************************************************;

%* OPTIONS NOMPRINT NOMERROR NOSERROR NOMLOGIC NOSYMBOLGEN NOMACROGEN;
%* OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
%* remove comment chars for testing ;

  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN %GOTO Finish;
  %ELSE %IF ( %UPCASE(&ForMyUse) = DEBUG ) %THEN
  %DO;
    OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
  %END;
  %ELSE %IF ( %UPCASE(&ForMyUse) = NODEBUG ) %THEN
  %DO;
    OPTIONS NOMPRINT NOMERROR NOSERROR NOMLOGIC NOSYMBOLGEN NOMACROGEN;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* LOCAL macro variables declarations ;
%*____________________________________;

 %LOCAL    /* Define internal macro variables explicitely local */
  MacName  /* Name of macro: RemBugNL */
  MacVs    /* Version number major.minor of macro: 0.3 */
  MaxName  /* maximum variable name length */
  MaxLabel /* maximum label contents length */
  HalfName /* maximum function name length */
  GenerDS  /* maximum generation dataset name length */
  NameLen  /* Length of name */
  Valid1st /* String of all valid characters in first position of name */
  ValidChr /* String of all valid characters in name */
  StartPos /* starting position in string to match with valid characters */
  Result   /* either Right or Wrong */
  Err      /* text 'E R R O R' as consecutive characters */
  Warn     /* text 'W A R N I N G' as consecutive characters */
  ;

  %LET MacName = SAS_name;
  %LET MacVs   = 0.3;
  %LET Err     = %SYSFUNC(COMPRESS(E R R O R, %STR( )));
  %LET Warn    = %SYSFUNC(COMPRESS(W A R N I N G, %STR( )));

%* Macro header in LOG file, dependent on value of CalledBy;
 %IF ( %UPCASE(&CalledBy) EQ %UPCASE(&MacName) ) %THEN
 %DO;
  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT |  Macro &MacName, vs. &MacVs, by Jim Groeneveld, 19 Mar 2003  |;
  %PUT |     Checking validity of SAS names like SCL`s SASname     |;
  %PUT |___________________________________________________________|;
  %PUT;

%* Feedback of specified arguments;
  %PUT &MacName: Name     = &Name    ;
  %PUT &MacName: Type     = &Type    ;
  %PUT &MacName: CalledBy = &CalledBy;
  %PUT &MacName: Version  = &Version ;
  %*PUT &MacName: ForMyUse = &ForMyUse;
 %END;

  %IF (&Version NE AND &Version NE &MacVs) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Specified version &Version does not match &MacVs,;
    %PUT .                       macro &MacName will abort;
    %GOTO Finish;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Adaptation of variable name length limits to SAS version (6 or >6) ;
%*____________________________________________________________________;

  %IF (%SCAN(&SYSVER,1,.) GT 6) %THEN
  %DO; %* vs. 7 and up;
    %IF (%UPCASE(%SYSFUNC(GETOPTION(VALIDVARNAME))) EQ V6) %THEN
    %DO;
      %LET MaxName = 8;
      %LET MaxLabel = 40;
      %LET HalfName = 8;
      %LET GenerDS = 8;
    %END;
    %ELSE /* V7 or UPCASE */
    %DO;
      %LET MaxName = 32;
      %LET MaxLabel = 256;
      %LET HalfName = 16;
      %LET GenerDS = 28;
    %END;
  %END;
  %ELSE
  %DO; %* any older vs. (6);
    %LET MaxName = 8;
    %LET MaxLabel = 40;
    %LET HalfName = 8;
    %LET GenerDS = 8;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Initializations and pre-calculations ;
%*______________________________________;

  %LET Valid1st = _ABCDEFGHIJKLMNOPQRSTUVWXYZ;
  %LET ValidChr = _ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789;
  %LET Name = %UPCASE (%BQUOTE (&Name) );
  %LET NameLen = %LENGTH (&Name);

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Core code, determining Right or Wrong ;
%*_______________________________________;

  %LET Result = Unknown;

  %IF ( ( %UPCASE(&Type) EQ FORMAT OR %UPCASE(&Type) EQ INFORMAT )
    AND %SUBSTR (&Name, 1, 1) EQ %STR($) ) %THEN %LET StartPos = 2;
  %ELSE %LET StartPos = 1;

  %IF (&NameLen LT &StartPos) %THEN %LET Result = Wrong;

  %ELSE %IF (%UPCASE(&Type) EQ VARIABLE
          OR %UPCASE(&Type) EQ ARRAY
          OR %UPCASE(&Type) EQ DATASET
          OR %UPCASE(&Type) EQ VIEW
          OR %UPCASE(&Type) EQ CATALOG
          OR %UPCASE(&Type) EQ INDEX
          OR %UPCASE(&Type) EQ ENTRY
          OR %UPCASE(&Type) EQ DSWINDOW
          OR %UPCASE(&Type) EQ MACRO
          OR %UPCASE(&Type) EQ MACVAR
          OR %UPCASE(&Type) EQ MCWINDOW
          OR %UPCASE(&Type) EQ SCLVAR
            ) %THEN
  %DO;
    %IF (&NameLen GT &MaxName) %THEN %LET Result = Wrong;
  %END;

  %ELSE %IF (%UPCASE(&Type) EQ VARLABEL
            ) %THEN
  %DO;
    %IF (&NameLen GT &MaxLabel) %THEN %LET Result = Wrong;
  %END;

  %ELSE %IF (%UPCASE(&Type) EQ GENERDS
            ) %THEN
  %DO;
    %IF (&NameLen GT &GenerDS) %THEN %LET Result = Wrong;
  %END;

  %ELSE %IF (%UPCASE(&Type) EQ FUNCTION
          OR %UPCASE(&Type) EQ CALLROUTINE
            ) %THEN
  %DO;
    %IF (&NameLen GT &HalfName) %THEN %LET Result = Wrong;
  %END;

  %ELSE %IF (%UPCASE(&Type) EQ FORMAT
          OR %UPCASE(&Type) EQ ENGINE
          OR %UPCASE(&Type) EQ LIBNAME
          OR %UPCASE(&Type) EQ FILENAME
            ) %THEN
  %DO;
    %IF (&NameLen GT 8) %THEN %LET Result = Wrong;
  %END;

  %ELSE %IF (%UPCASE(&Type) EQ INFORMAT
            ) %THEN
  %DO;
    %IF (&NameLen GT 7) %THEN %LET Result = Wrong;
  %END;

  %ELSE %LET Result = Unmatched Type; %* unrecognized name type;


  %IF (&Result EQ Wrong) %THEN /* nothing, just skip the rest */ ;

%* Check last (in)format character, which may not be a digit;
  %ELSE %IF ( ( %UPCASE(&Type) EQ FORMAT OR %UPCASE(&Type) EQ INFORMAT )
    AND %INDEX (&Valid1st, %SUBSTR (&Name, &Namelen, 1) ) EQ 0 ) %THEN
      %LET Result = Wrong; %* Last char of (in)format may not be a digit;

  %ELSE %IF ( %INDEX (&Valid1st, %SUBSTR (&Name, &StartPos, 1) ) EQ 0 ) %THEN
                            %LET Result = Wrong;

  %ELSE %IF ( %LENGTH (%SYSFUNC (COMPRESS (%SUBSTR (&Name, &StartPos), &ValidChr) ) )
         GT 0 ) %THEN %LET Result = Wrong;

  %ELSE %IF ( %INDEX (&Name, %STR( )) ) %THEN %LET Result = Wrong;

  %ELSE %IF (&Result EQ Unknown) %THEN %LET Result = Right;
%* thus if it is Unmatched Type it remains that way;

  &Result /* (without semicolon) being passed to the calling program code */

  %IF ( %UPCASE(&ForMyUse) = DEBUG ) %THEN
  %DO; %* finish debugging, reset options to no debug;
    OPTIONS NOMPRINT NOMERROR NOSERROR NOMLOGIC NOSYMBOLGEN NOMACROGEN;
  %END;

%Finish:

%*~~~~~~~~~~~~~;
%MEND SAS_name ;
%*_____________;

